@@ -1,8 +1,10 @@
 # SPDX-License-Identifier: MIT
 
+from __future__ import annotations
+
 import re
 
-from typing import Any, Dict, List, Tuple, Union
+from typing import Any, Dict, List, Tuple
 
 import attr
 import attrs
@@ -49,12 +51,12 @@ CC(a=1)
 
 @attr.s
 class DD:
-    x: List[int] = attr.ib()
+    x: list[int] = attr.ib()
 
 
 @attr.s
 class EE:
-    y: "List[int]" = attr.ib()
+    y: "list[int]" = attr.ib()
 
 
 @attr.s
@@ -176,7 +178,7 @@ class Validated:
             attr.validators.instance_of(C), attr.validators.instance_of(list)
         ),
     )
-    a = attr.ib(
+    aa = attr.ib(
         type=Tuple[C],
         validator=attr.validators.deep_iterable(
             attr.validators.instance_of(C), attr.validators.instance_of(tuple)
@@ -210,10 +212,8 @@ class Validated:
     # Test different forms of instance_of
     g: int = attr.ib(validator=attr.validators.instance_of(int))
     h: int = attr.ib(validator=attr.validators.instance_of((int,)))
-    j: Union[int, str] = attr.ib(
-        validator=attr.validators.instance_of((int, str))
-    )
-    k: Union[int, str, C] = attr.ib(
+    j: int | str = attr.ib(validator=attr.validators.instance_of((int, str)))
+    k: int | str | C = attr.ib(
         validator=attrs.validators.instance_of((int, C, str))
     )
 
@@ -324,14 +324,14 @@ class ValidatedSetter2:
 
 
 # field_transformer
-def ft_hook(cls: type, attribs: List[attr.Attribute]) -> List[attr.Attribute]:
+def ft_hook(cls: type, attribs: list[attr.Attribute]) -> list[attr.Attribute]:
     return attribs
 
 
 # field_transformer
 def ft_hook2(
-    cls: type, attribs: List[attrs.Attribute]
-) -> List[attrs.Attribute]:
+    cls: type, attribs: list[attrs.Attribute]
+) -> list[attrs.Attribute]:
     return attribs
 
 
@@ -395,16 +395,16 @@ class MRO:
 
 @attr.s
 class FactoryTest:
-    a: List[int] = attr.ib(default=attr.Factory(list))
-    b: List[Any] = attr.ib(default=attr.Factory(list, False))
-    c: List[int] = attr.ib(default=attr.Factory((lambda s: s.a), True))
+    a: list[int] = attr.ib(default=attr.Factory(list))
+    b: list[Any] = attr.ib(default=attr.Factory(list, False))
+    c: list[int] = attr.ib(default=attr.Factory((lambda s: s.a), True))
 
 
 @attrs.define
 class FactoryTest2:
-    a: List[int] = attrs.field(default=attrs.Factory(list))
-    b: List[Any] = attrs.field(default=attrs.Factory(list, False))
-    c: List[int] = attrs.field(default=attrs.Factory((lambda s: s.a), True))
+    a: list[int] = attrs.field(default=attrs.Factory(list))
+    b: list[Any] = attrs.field(default=attrs.Factory(list, False))
+    c: list[int] = attrs.field(default=attrs.Factory((lambda s: s.a), True))
 
 
 attrs.asdict(FactoryTest2())
@@ -468,3 +468,8 @@ if attrs.has(foo) or attr.has(foo):
 @attrs.define(unsafe_hash=True)
 class Hashable:
     pass
+
+
+def test(cls: type) -> None:
+    if attr.has(cls):
+        attr.resolve_types(cls)
